==> AlwaysMoreItems-1.0.1-sources.jar: net/glasslauncher/mods/alwaysmoreitems/util/StackHelper.java <==
package net.glasslauncher.mods.alwaysmoreitems.util;

import net.fabricmc.api.EnvType;
import net.fabricmc.loader.api.FabricLoader;
import net.glasslauncher.mods.alwaysmoreitems.api.AMINbt;
import net.glasslauncher.mods.alwaysmoreitems.api.SubProvider;
import net.glasslauncher.mods.alwaysmoreitems.gui.widget.ingredients.IGuiIngredient;
import net.minecraft.class_124;
import net.minecraft.class_133;
import net.minecraft.class_31;
import net.minecraft.class_71;
import net.minecraft.class_8;
import net.modificationstation.stationapi.api.registry.ItemRegistry;
import net.modificationstation.stationapi.api.util.Identifier;

import javax.annotation.*;
import java.util.*;

public class StackHelper implements net.glasslauncher.mods.alwaysmoreitems.api.recipe.StackHelper {
	/**
	 * Returns a list of items in slots that complete the recipe defined by requiredStacksList.
	 * Returns a result that contains missingItems if there are not enough items in availableItemStacks.
	 */
	@Nonnull
	public MatchingItemsResult getMatchingItems(@Nonnull List<class_31> availableItemStacks, @Nonnull Map<Integer, ? extends IGuiIngredient<class_31>> ingredientsMap) {
		MatchingItemsResult matchingItemResult = new MatchingItemsResult();

		int recipeSlotNumber = -1;
		SortedSet<Integer> keys = new TreeSet<>(ingredientsMap.keySet());
		for (Integer key : keys) {
			IGuiIngredient<class_31> ingredient = ingredientsMap.get(key);
			if (!ingredient.isInput()) {
				continue;
			}
			recipeSlotNumber++;

			List<class_31> requiredStacks = ingredient.getAll();
			if (requiredStacks.isEmpty()) {
				continue;
			}

			class_31 matching = containsStack(availableItemStacks, requiredStacks);
			if (matching == null) {
				matchingItemResult.missingItems.add(key);
			} else {
				class_31 matchingSplit = matching.method_695(1);
				if (matching.field_751 == 0) {
					availableItemStacks.remove(matching);
				}
				matchingItemResult.matchingItems.put(recipeSlotNumber, matchingSplit);
			}
		}

		return matchingItemResult;
	}

	@Nullable
	public class_133 getSlotWithStack(@Nonnull class_71 container, @Nonnull Iterable<Integer> slotNumbers, @Nonnull class_31 stack) {
		StackHelper stackHelper = AlwaysMoreItems.getStackHelper();

		for (Integer slotNumber : slotNumbers) {
			class_133 slot = container.method_2084(slotNumber);
			if (slot != null) {
				class_31 slotStack = slot.method_472();
				if (stackHelper.isIdentical(stack, slotStack)) {
					return slot;
				}
			}
		}
		return null;
	}

	@Nonnull
	public List<class_31> removeDuplicateItemStacks(@Nonnull Iterable<class_31> stacks) {
		List<class_31> newStacks = new ArrayList<>();
		for (class_31 stack : stacks) {
			if (stack != null && containsStack(newStacks, stack) == null) {
				newStacks.add(stack);
			}
		}
		return newStacks;
	}

	/* Returns an ItemStack from "stacks" if it isIdentical to an ItemStack from "contains" */
	@Nullable
	public class_31 containsStack(@Nullable Iterable<class_31> stacks, @Nullable Iterable<class_31> contains) {
		if (stacks == null || contains == null) {
			return null;
		}

		for (class_31 containStack : contains) {
			class_31 matchingStack = containsStack(stacks, containStack);
			if (matchingStack != null) {
				return matchingStack;
			}
		}

		return null;
	}

	/* Returns an ItemStack from "stacks" if it isIdentical to "contains" */
	@Nullable
	public class_31 containsStack(@Nullable Iterable<class_31> stacks, @Nullable class_31 contains) {
		if (stacks == null || contains == null) {
			return null;
		}

		for (class_31 stack : stacks) {
			if (isIdentical(contains, stack)) {
				return stack;
			}
		}
		return null;
	}

	public boolean isIdentical(@Nullable class_31 lhs, @Nullable class_31 rhs) {
		if (lhs == rhs) {
			return true;
		}

		if (lhs == null || rhs == null) {
			return false;
		}

		if (lhs.method_694() != rhs.method_694()) {
			return false;
		}

		if (lhs.method_722() != -1/*OreDictionary.WILDCARD_VALUE*/) {
			if (lhs.method_722() != rhs.method_722()) {
				return false;
			}
		}

		return lhs.method_702(rhs);
	}

	@Override
	@Nonnull
	public List<class_31> getSubtypes(@Nullable class_31 itemStack) {
		if (itemStack == null) {
			AlwaysMoreItems.LOGGER.error("Null itemStack", new NullPointerException());
			return Collections.emptyList();
		}

		class_124 item = itemStack.method_694();
		if (item == null) {
			AlwaysMoreItems.LOGGER.error("Null item in itemStack", new NullPointerException());
			return Collections.emptyList();
		}

		if (itemStack.method_722() != -1/*OreDictionary.WILDCARD_VALUE*/) {
			return new ArrayList<>() {
				{
					add(itemStack);
				}
			};
		}

		return getSubtypes(item, itemStack.field_751);
	}

	@Nonnull
	public List<class_31> getSubtypes(@Nonnull class_124 item, int stackSize) {
//		List<ItemStack> itemStacks = new ArrayList<>();

//		for (CreativeTabs itemTab : item.getCreativeTabs()) {
//			List<ItemStack> subItems = new ArrayList<>();
//			item.getSubItems(item, itemTab, subItems);
//			for (ItemStack subItem : subItems) {
//				if (subItem.stackSize != stackSize) {
//					ItemStack subItemCopy = subItem.copy();
//					subItemCopy.stackSize = stackSize;
//					itemStacks.add(subItemCopy);
//				} else {
//					itemStacks.add(subItem);
//				}
//			}
//		}
		List<class_31> subItems = ((SubProvider) item).getSubItems().stream().peek(itemStack -> itemStack.field_751 = stackSize).toList();

		if (subItems.isEmpty()) {
			subItems = new ArrayList<>();
			List<String> keyCache = new ArrayList<>();
			if (FabricLoader.getInstance().getEnvironmentType().equals(EnvType.SERVER)) {
				subItems.add(new class_31(item, stackSize, 0));
				return subItems;
			}
			for (int i = 0; i < 16; i++) {
				try { // Shitcoders go brrr
					class_31 itemStack = new class_31(item, stackSize, i);
					String translationKey = itemStack.method_726();
					if (keyCache.contains(translationKey)) {
						break;
					}
					keyCache.add(translationKey);
					subItems.add(itemStack);
				}
				catch (Exception e) {
                    //noinspection OptionalGetWithoutIsPresent if we throw an exception on this get, then someone fucked up big time.
                    AlwaysMoreItems.LOGGER.error("An item being autoregistered threw an exception, yell at the creator of " + ItemRegistry.INSTANCE.getId(item.field_461).get(), e);
				}
			}
		}

		return subItems;
	}

	@Override
	@Nonnull
	public List<class_31> getAllSubtypes(@Nullable Iterable stacks) {
		if (stacks == null) {
			AlwaysMoreItems.LOGGER.error("Null stacks", new NullPointerException());
			return Collections.emptyList();
		}

		List<class_31> allSubtypes = new ArrayList<>();
		getAllSubtypes(allSubtypes, stacks);
		return allSubtypes;
	}

	private void getAllSubtypes(@Nonnull List<class_31> subtypesList, @Nonnull Iterable stacks) {
		for (Object obj : stacks) {
			if (obj instanceof class_31) {
				class_31 itemStack = (class_31) obj;
				List<class_31> subtypes = getSubtypes(itemStack);
				subtypesList.addAll(subtypes);
			} else if (obj instanceof Iterable) {
				getAllSubtypes(subtypesList, (Iterable) obj);
			} else if (obj != null) {
				AlwaysMoreItems.LOGGER.error("Unknown object found: {}", obj);
			}
		}
	}

	@Override
	@Nonnull
	public List<class_31> toItemStackList(@Nullable Object stacks) {
		if (stacks == null) {
			return Collections.emptyList();
		}

		List<class_31> itemStacksList = new ArrayList<>();
		toItemStackList(itemStacksList, stacks);
		return removeDuplicateItemStacks(itemStacksList);
	}

	private void toItemStackList(@Nonnull List<class_31> itemStackList, @Nullable Object input) {
		if (input instanceof class_31) {
			itemStackList.add((class_31) input);
		} else if (input instanceof String) {
//			List<ItemStack> stacks = ItemRegistry.INSTANCE.getEntryList(TagKey.of(ItemRegistry.KEY, Identifier.of(string)));
//			itemStackList.addAll(stacks);
		} else if (input instanceof Iterable) {
			for (Object obj : (Iterable) input) {
				toItemStackList(itemStackList, obj);
			}
		} else if (input != null) {
			AlwaysMoreItems.LOGGER.error("Unknown object found: {}", input);
		}
	}

	@Nonnull
	public String getUniqueIdentifierForStack(@Nonnull class_31 stack) {
		return getUniqueIdentifierForStack(stack, false);
	}

	@Nonnull
	public String getUniqueIdentifierForStack(@Nonnull class_31 stack, boolean wildcard) {
		class_124 item = stack.method_694();
		if (item == null) {
			throw new ItemUidException("Found an itemStack with a null item. This is an error from another mod.");
		}

		Identifier itemName = ItemRegistry.INSTANCE.getId(item);
		if (itemName == null) {
			throw new ItemUidException("No name for item in GameData itemRegistry: " + item.getClass());
		}

		String itemNameString = itemName.toString();
		int metadata = stack.method_722();
		if (wildcard || metadata == -1 /*OreDictionary.WILDCARD_VALUE*/) {
			return itemNameString;
		}

		StringBuilder itemKey = new StringBuilder(itemNameString);
		if (stack.method_719()) {
			itemKey.append(':').append(metadata);
			if (!((AMINbt) stack.getStationNbt()).always_More_Items$hasNoTags()) {
				class_8 nbtTagCompound = AlwaysMoreItems.getHelpers().getNbtIgnoreList().getNbt(stack);
				if (nbtTagCompound != null && !((AMINbt) nbtTagCompound).always_More_Items$hasNoTags()) {
					itemKey.append(':').append(nbtTagCompound);
				}
			}
		}

		return itemKey.toString();
	}

	@Nonnull
	public List<String> getUniqueIdentifiersWithWildcard(@Nonnull class_31 itemStack) {
		String uid = getUniqueIdentifierForStack(itemStack, false);
		String uidWild = getUniqueIdentifierForStack(itemStack, true);

		if (uid.equals(uidWild)) {
			return Collections.singletonList(uid);
		} else {
			return Arrays.asList(uid, uidWild);
		}
	}

	public int addStack(@Nonnull class_71 container, @Nonnull Collection<Integer> slotIndexes, @Nonnull class_31 stack, boolean doAdd) {
		int added = 0;
		// Add to existing stacks first
		for (Integer slotIndex : slotIndexes) {
			class_133 slot = container.method_2084(slotIndex);
			if (slot == null) {
				continue;
			}

			class_31 inventoryStack = slot.method_472();
			if (inventoryStack == null || inventoryStack.method_694() == null) {
				continue;
			}

			// Already occupied by different item, skip this slot.
			if (!inventoryStack.method_715() || !inventoryStack.method_702(stack) || !class_31.method_703(inventoryStack, stack)) {
				continue;
			}

			int remain = stack.field_751 - added;
			int space = inventoryStack.method_709() - inventoryStack.field_751;
			if (space <= 0) {
				continue;
			}

			// Enough space
			if (space >= remain) {
				if (doAdd) {
					inventoryStack.field_751 += remain;
				}
				return stack.field_751;
			}

			// Not enough space
			if (doAdd) {
				inventoryStack.field_751 = inventoryStack.method_709();
			}

			added += space;
		}

		if (added >= stack.field_751) {
			return added;
		}

		for (Integer slotIndex : slotIndexes) {
			class_133 slot = container.method_2084(slotIndex);
			if (slot == null) {
				continue;
			}

			class_31 inventoryStack = slot.method_472();
			if (inventoryStack != null) {
				continue;
			}

			if (doAdd) {
				class_31 stackToAdd = stack.method_724();
				stackToAdd.field_751 = stack.field_751 - added;
				slot.method_479(stackToAdd);
			}
			return stack.field_751;
		}

		return added;
	}

	public static class MatchingItemsResult {
		@Nonnull
		public final Map<Integer, class_31> matchingItems = new HashMap<>();
		@Nonnull
		public final List<Integer> missingItems = new ArrayList<>();
	}
}

==> AlwaysMoreItems-1.1.0-sources.jar: net/glasslauncher/mods/alwaysmoreitems/util/StackHelper.java <==
package net.glasslauncher.mods.alwaysmoreitems.util;

import net.fabricmc.api.EnvType;
import net.fabricmc.loader.api.FabricLoader;
import net.glasslauncher.mods.alwaysmoreitems.api.AMINbt;
import net.glasslauncher.mods.alwaysmoreitems.api.SubItemHelper;
import net.glasslauncher.mods.alwaysmoreitems.api.SubItemProvider;
import net.glasslauncher.mods.alwaysmoreitems.config.AMIConfig;
import net.glasslauncher.mods.alwaysmoreitems.gui.widget.ingredients.IGuiIngredient;
import net.minecraft.class_124;
import net.minecraft.class_133;
import net.minecraft.class_31;
import net.minecraft.class_71;
import net.minecraft.class_8;
import net.modificationstation.stationapi.api.registry.ItemRegistry;
import net.modificationstation.stationapi.api.util.Identifier;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.*;

public class StackHelper implements net.glasslauncher.mods.alwaysmoreitems.api.recipe.StackHelper {
	/**
	 * Returns a list of items in slots that complete the recipe defined by requiredStacksList.
	 * Returns a result that contains missingItems if there are not enough items in availableItemStacks.
	 */
	@Nonnull
	public MatchingItemsResult getMatchingItems(@Nonnull List<class_31> availableItemStacks, @Nonnull Map<Integer, ? extends IGuiIngredient<class_31>> ingredientsMap) {
		MatchingItemsResult matchingItemResult = new MatchingItemsResult();

		int recipeSlotNumber = -1;
		SortedSet<Integer> keys = new TreeSet<>(ingredientsMap.keySet());
		for (Integer key : keys) {
			IGuiIngredient<class_31> ingredient = ingredientsMap.get(key);
			if (!ingredient.isInput()) {
				continue;
			}
			recipeSlotNumber++;

			List<class_31> requiredStacks = ingredient.getAll();
			if (requiredStacks.isEmpty()) {
				continue;
			}

			class_31 matching = containsStack(availableItemStacks, requiredStacks);
			if (matching == null) {
				matchingItemResult.missingItems.add(key);
			} else {
				class_31 matchingSplit = matching.method_695(1);
				if (matching.field_751 == 0) {
					availableItemStacks.remove(matching);
				}
				matchingItemResult.matchingItems.put(recipeSlotNumber, matchingSplit);
			}
		}

		return matchingItemResult;
	}

	@Nullable
	public class_133 getSlotWithStack(@Nonnull class_71 container, @Nonnull Iterable<Integer> slotNumbers, @Nonnull class_31 stack) {
		StackHelper stackHelper = AlwaysMoreItems.getStackHelper();

		for (Integer slotNumber : slotNumbers) {
			class_133 slot = container.method_2084(slotNumber);
			if (slot != null) {
				class_31 slotStack = slot.method_472();
				if (stackHelper.isIdentical(stack, slotStack)) {
					return slot;
				}
			}
		}
		return null;
	}

	@Nonnull
	public List<class_31> removeDuplicateItemStacks(@Nonnull Iterable<class_31> stacks) {
		List<class_31> newStacks = new ArrayList<>();
		for (class_31 stack : stacks) {
			if (stack != null && containsStack(newStacks, stack) == null) {
				newStacks.add(stack);
			}
		}
		return newStacks;
	}

	/* Returns an ItemStack from "stacks" if it isIdentical to an ItemStack from "contains" */
	@Nullable
	public class_31 containsStack(@Nullable Iterable<class_31> stacks, @Nullable Iterable<class_31> contains) {
		if (stacks == null || contains == null) {
			return null;
		}

		for (class_31 containStack : contains) {
			class_31 matchingStack = containsStack(stacks, containStack);
			if (matchingStack != null) {
				return matchingStack;
			}
		}

		return null;
	}

	/* Returns an ItemStack from "stacks" if it isIdentical to "contains" */
	@Nullable
	public class_31 containsStack(@Nullable Iterable<class_31> stacks, @Nullable class_31 contains) {
		if (stacks == null || contains == null) {
			return null;
		}

		for (class_31 stack : stacks) {
			if (isIdentical(contains, stack)) {
				return stack;
			}
		}
		return null;
	}

	public boolean isIdentical(@Nullable class_31 lhs, @Nullable class_31 rhs) {
		if (lhs == rhs) {
			return true;
		}

		if (lhs == null || rhs == null) {
			return false;
		}

		if (lhs.method_694() != rhs.method_694()) {
			return false;
		}

		if (lhs.method_722() != -1/*OreDictionary.WILDCARD_VALUE*/) {
			if (lhs.method_722() != rhs.method_722()) {
				return false;
			}
		}

		return lhs.method_702(rhs);
	}

	@Override
	@Nonnull
	public List<class_31> getSubtypes(@Nullable class_31 itemStack) {
		if (itemStack == null) {
			AlwaysMoreItems.LOGGER.error("Null itemStack", new NullPointerException());
			return Collections.emptyList();
		}

		class_124 item = itemStack.method_694();
		if (item == null) {
			AlwaysMoreItems.LOGGER.error("Null item in itemStack", new NullPointerException());
			return Collections.emptyList();
		}

		if (itemStack.method_722() != -1/*OreDictionary.WILDCARD_VALUE*/) {
			return new ArrayList<>() {
				{
					add(itemStack);
				}
			};
		}

		return getSubtypes(item, itemStack.field_751);
	}

	@Nonnull
	public List<class_31> getSubtypes(@Nonnull class_124 item, int stackSize) {
//		List<ItemStack> itemStacks = new ArrayList<>();

//		for (CreativeTabs itemTab : item.getCreativeTabs()) {
//			List<ItemStack> subItems = new ArrayList<>();
//			item.getSubItems(item, itemTab, subItems);
//			for (ItemStack subItem : subItems) {
//				if (subItem.stackSize != stackSize) {
//					ItemStack subItemCopy = subItem.copy();
//					subItemCopy.stackSize = stackSize;
//					itemStacks.add(subItemCopy);
//				} else {
//					itemStacks.add(subItem);
//				}
//			}
//		}
		List<class_31> subItems = SubItemHelper.getSubItems(item);

		if (subItems != null && !subItems.isEmpty()) {
			subItems = subItems.stream().peek(itemStack -> itemStack.field_751 = stackSize).toList();
		}
		else {
			subItems = new ArrayList<>();
			List<String> keyCache = new ArrayList<>();
			if (FabricLoader.getInstance().getEnvironmentType().equals(EnvType.SERVER)) {
				subItems.add(new class_31(item, stackSize, 0));
				return subItems;
			}
			for (int i = 0; i < 16; i++) {
				try { // Shitcoders go brrr
					class_31 itemStack = new class_31(item, stackSize, i);
					String translationKey = itemStack.method_726();
					if (keyCache.contains(translationKey) || (!AMIConfig.ignoreBadNames() && Objects.equals(translationKey, itemStack.method_694().method_469()))) {
						break;
					}
					keyCache.add(translationKey);
					subItems.add(itemStack);
				}
				catch (Exception e) {
                    //noinspection OptionalGetWithoutIsPresent if we throw an exception on this get, then someone fucked up big time.
                    AlwaysMoreItems.LOGGER.error("An item being autoregistered threw an exception, yell at the creator of " + ItemRegistry.INSTANCE.getId(item.field_461).get(), e);
				}
			}
		}

		return subItems;
	}

	@Override
	@Nonnull
	public List<class_31> getAllSubtypes(@Nullable Iterable stacks) {
		if (stacks == null) {
			AlwaysMoreItems.LOGGER.error("Null stacks", new NullPointerException());
			return Collections.emptyList();
		}

		List<class_31> allSubtypes = new ArrayList<>();
		getAllSubtypes(allSubtypes, stacks);
		return allSubtypes;
	}

	private void getAllSubtypes(@Nonnull List<class_31> subtypesList, @Nonnull Iterable stacks) {
		for (Object obj : stacks) {
			if (obj instanceof class_31) {
				class_31 itemStack = (class_31) obj;
				List<class_31> subtypes = getSubtypes(itemStack);
				subtypesList.addAll(subtypes);
			} else if (obj instanceof Iterable) {
				getAllSubtypes(subtypesList, (Iterable) obj);
			} else if (obj != null) {
				AlwaysMoreItems.LOGGER.error("Unknown object found: {}", obj);
			}
		}
	}

	@Override
	@Nonnull
	public List<class_31> toItemStackList(@Nullable Object stacks) {
		if (stacks == null) {
			return Collections.emptyList();
		}

		List<class_31> itemStacksList = new ArrayList<>();
		toItemStackList(itemStacksList, stacks);
		return removeDuplicateItemStacks(itemStacksList);
	}

	private void toItemStackList(@Nonnull List<class_31> itemStackList, @Nullable Object input) {
		if (input instanceof class_31) {
			itemStackList.add((class_31) input);
		} else if (input instanceof String) {
//			List<ItemStack> stacks = ItemRegistry.INSTANCE.getEntryList(TagKey.of(ItemRegistry.KEY, Identifier.of(string)));
//			itemStackList.addAll(stacks);
		} else if (input instanceof Iterable) {
			for (Object obj : (Iterable) input) {
				toItemStackList(itemStackList, obj);
			}
		} else if (input != null) {
			AlwaysMoreItems.LOGGER.error("Unknown object found: {}", input);
		}
	}

	@Nonnull
	public String getUniqueIdentifierForStack(@Nonnull class_31 stack) {
		return getUniqueIdentifierForStack(stack, false);
	}

	@Nonnull
	public String getUniqueIdentifierForStack(@Nonnull class_31 stack, boolean wildcard) {
		class_124 item = stack.method_694();
		if (item == null) {
			throw new ItemUidException("Found an itemStack with a null item. This is an error from another mod.");
		}

		Identifier itemName = ItemRegistry.INSTANCE.getId(item);
		if (itemName == null) {
			throw new ItemUidException("No name for item in GameData itemRegistry: " + item.getClass());
		}

		String itemNameString = itemName.toString();
		int metadata = stack.method_722();
		if (wildcard || metadata == -1 /*OreDictionary.WILDCARD_VALUE*/) {
			return itemNameString;
		}

		StringBuilder itemKey = new StringBuilder(itemNameString);
		if (stack.method_719()) {
			itemKey.append(':').append(metadata);
			if (!((AMINbt) stack.getStationNbt()).always_More_Items$hasNoTags()) {
				class_8 nbtTagCompound = AlwaysMoreItems.getHelpers().getNbtIgnoreList().getNbt(stack);
				if (nbtTagCompound != null && !((AMINbt) nbtTagCompound).always_More_Items$hasNoTags()) {
					itemKey.append(':').append(nbtTagCompound);
				}
			}
		}

		return itemKey.toString();
	}

	@Nonnull
	public List<String> getUniqueIdentifiersWithWildcard(@Nonnull class_31 itemStack) {
		String uid = getUniqueIdentifierForStack(itemStack, false);
		String uidWild = getUniqueIdentifierForStack(itemStack, true);

		if (uid.equals(uidWild)) {
			return Collections.singletonList(uid);
		} else {
			return Arrays.asList(uid, uidWild);
		}
	}

	public int addStack(@Nonnull class_71 container, @Nonnull Collection<Integer> slotIndexes, @Nonnull class_31 stack, boolean doAdd) {
		int added = 0;
		// Add to existing stacks first
		for (Integer slotIndex : slotIndexes) {
			class_133 slot = container.method_2084(slotIndex);
			if (slot == null) {
				continue;
			}

			class_31 inventoryStack = slot.method_472();
			if (inventoryStack == null || inventoryStack.method_694() == null) {
				continue;
			}

			// Already occupied by different item, skip this slot.
			if (!inventoryStack.method_715() || !inventoryStack.method_702(stack) || !class_31.method_703(inventoryStack, stack)) {
				continue;
			}

			int remain = stack.field_751 - added;
			int space = inventoryStack.method_709() - inventoryStack.field_751;
			if (space <= 0) {
				continue;
			}

			// Enough space
			if (space >= remain) {
				if (doAdd) {
					inventoryStack.field_751 += remain;
				}
				return stack.field_751;
			}

			// Not enough space
			if (doAdd) {
				inventoryStack.field_751 = inventoryStack.method_709();
			}

			added += space;
		}

		if (added >= stack.field_751) {
			return added;
		}

		for (Integer slotIndex : slotIndexes) {
			class_133 slot = container.method_2084(slotIndex);
			if (slot == null) {
				continue;
			}

			class_31 inventoryStack = slot.method_472();
			if (inventoryStack != null) {
				continue;
			}

			if (doAdd) {
				class_31 stackToAdd = stack.method_724();
				stackToAdd.field_751 = stack.field_751 - added;
				slot.method_479(stackToAdd);
			}
			return stack.field_751;
		}

		return added;
	}

	public static class MatchingItemsResult {
		@Nonnull
		public final Map<Integer, class_31> matchingItems = new HashMap<>();
		@Nonnull
		public final List<Integer> missingItems = new ArrayList<>();
	}
}
